#!/usr/bin/env python

from avahihelper import AvahiBrowser

try:
    import rlcompleter
    import readline
    readline.parse_and_bind("tab: complete")
except Exception:
    pass

import gtk

# Thanks to Sandino Flores!!!
def connect(widget, signal_name):
    def decorator(function):
        callback = lambda widget, *args: function(*args)
        handler_id = widget.connect(signal_name, callback)
        function.func_globals["handler_id"] = handler_id
        return function
    return decorator

########### HELPER CLASSES ######################

class Entry(gtk.Entry):
    def __init__(self, text = None):
        gtk.Entry.__init__(self)
        self.set_text(text)

class CoolTreeview(gtk.TreeView):
    """ A multicolumn icon/text treeview. Yea! """
    
    def __init__(self, names, icons = {}, sorted = None, markup = False):
        gtk.TreeView.__init__(self)
        
        self.set_enable_search(False)
        self.set_show_expanders(True)
        for i in range(len(names)):
            column = gtk.TreeViewColumn(names[i])
            column.set_resizable(True)
            self.append_column(column)
            if icons.has_key(i):
                cell = gtk.CellRendererPixbuf()
                column.pack_start(cell, False)
                column.add_attribute(cell, 'pixbuf', icons[i])
                cell = gtk.CellRendererText()
                column.pack_start(cell, True)
                if markup:
                    column.add_attribute(cell, 'markup', i)
                else:
                    column.add_attribute(cell, 'text', i)
            else:
                cell = gtk.CellRendererText()
                column.pack_start(cell, True)
                if markup:
                    column.add_attribute(cell, 'markup', i)
                else:
                    column.add_attribute(cell, 'text', i)
            if not sorted is None:
                if sorted.has_key(i):
                    column.set_sort_column_id(sorted[i])
                else:
                    column.set_sort_column_id(i)


########### GTK WIDGETS ######################

window = gtk.Window(gtk.WINDOW_TOPLEVEL)
window.set_title("avahihelper test!")

# Browsing
label_type = gtk.Label("Service Type:")
entry_type = Entry("_workstation._tcp")
button_browse = gtk.Button(label="Add Browser")
button_unbrowse = gtk.Button(label="Remove Browser")
hbox_browse = gtk.HBox()
hbox_browse.pack_start(label_type, False)
hbox_browse.pack_start(entry_type, True)
hbox_browse.pack_end(button_unbrowse, False)
hbox_browse.pack_end(button_browse, False)

# Publishing
label_name = gtk.Label("Service Name: ")
entry_name = Entry("A Sample Service Name")
label_port = gtk.Label("Port: ")
entry_port = Entry("%s" % 5)
button_publish = gtk.Button(label="Publish")
button_unpublish = gtk.Button(label="Unpublish")
hbox_publish = gtk.HBox()
hbox_publish.pack_start(label_name, False)
hbox_publish.pack_start(entry_name, True)
hbox_publish.pack_start(label_port, False)
hbox_publish.pack_start(entry_port, True)
hbox_publish.pack_end(button_unpublish, False)
hbox_publish.pack_end(button_publish, False)

# Browser Tree
store_view = gtk.TreeStore(str,     # stype and number of services found 
                           str,     # name
                           str,     # address
                           str,     # port
                           object,  # associated AvahiBrowser instance
                           str)     # stype (without decorations)
sorted_view = gtk.TreeModelSort(store_view) 
treeview_view = CoolTreeview(["Type", "Name", "Address", "Port"], {}, {}, True)
treeview_view.set_model(sorted_view)
scrolledwindow_view = gtk.ScrolledWindow()
scrolledwindow_view.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
scrolledwindow_view.add(treeview_view)

# StatusBar
statusbar = gtk.Statusbar()
statusbar.m_id = None
statusbar.c_id = statusbar.get_context_id("")

# pack and show window
vbox = gtk.VBox()
vbox.pack_start(hbox_browse, False)
vbox.pack_start(hbox_publish, False)
vbox.pack_start(scrolledwindow_view, True)
vbox.pack_end(statusbar, False)
window.set_size_request(640, 480)
window.add(vbox)
window.show_all()

import dbus
from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)
bus = dbus.SystemBus()

########### CORE LOGIC AND EVENT HANDLING ######################

def statusbar_message(text):
    if statusbar.m_id:
        statusbar.pop(statusbar.c_id)
    statusbar.m_id = statusbar.push(statusbar.c_id, text or "")

def browse_started(avahi_browser):
    row = store_view[avahi_browser.iter]
    row[0] = "<b>%s (%d)</b>" % (row[-1], avahi_browser.len())

def browse_found(avahi_browser, avahi_service):
    r = [avahi_service.stype,
         avahi_service.name,
         avahi_service.addr,
         avahi_service.port,
         avahi_service,
         None]
    iter = store_view.append(avahi_browser.iter, r)
    avahi_service.iter = iter
    row = store_view[avahi_browser.iter]
    row[0] = "<b>%s (%d)</b>" % (row[-1], avahi_browser.len())
    i = sorted_view.convert_child_iter_to_iter(None, avahi_browser.iter)
    p = sorted_view.get_path(i)
    treeview_view.expand_row(p, True)

def browse_gone(avahi_browser, avahi_service):
    row = store_view[avahi_browser.iter]
    row[0] = "<b>%s (%d)</b>" % (row[-1], avahi_browser.len())
    del store_view[avahi_service.iter]

@connect(entry_type, 'activate')
@connect(button_browse, 'clicked')
def add_browse():
    global row
    stype = str(entry_type.get_text())
    for row in store_view:
        if row[-1] == stype:
            statusbar_message("WILL NOT WATCH AGAIN: "\
                              "This type of service is already being watched")
            return
    avahi_browser = AvahiBrowser(bus, stype, False)
    r = ["<i>%s ...</i>" % stype, None, None, None, avahi_browser, stype]
    iter = store_view.append(None, r)
    statusbar_message("Added watch for %s." % stype)
    avahi_browser.iter = iter
    avahi_browser.connect('started', browse_started)
    avahi_browser.connect('found', browse_found)
    avahi_browser.connect('gone', browse_gone)
    avahi_browser.browse()
    
    update_sensitive()
    entry_name.grab_focus()


@connect(entry_type, 'changed')
def type_changed():
    stype = entry_type.get_text() or ""
    button_browse.set_sensitive(len(stype) != 0)
    update_sensitive()

def update_sensitive():
    stype = entry_type.get_text() or ""
    s = False
    for row in store_view:
        if row[-1] == stype:
            s = (True)
            break
    button_browse.set_sensitive(not s)
    button_unbrowse.set_sensitive(s)
    hbox_publish.set_sensitive(s)
    button_unpublish.set_sensitive(False) #FIXME

update_sensitive()
statusbar_message("Welcome!")
window.connect('destroy', gtk.main_quit)
gtk.main()
